@@ -6,7 +6,8 @@
 #include <getopt.h>
 
 #include <ip6tables.h>
-#include <linux/netfilter_ipv6/ip6t_mark.h>
+/* For 64bit kernel / 32bit userspace */
+#include "../include/linux/netfilter_ipv6/ip6t_mark.h"
 
 /* Function which prints out usage message. */
 static void
@@ -16,7 +17,7 @@ help(void)
 "MARK match v%s options:\n"
 "[!] --mark value[/mask]         Match nfmark value with optional mask\n"
 "\n",
-NETFILTER_VERSION);
+IPTABLES_VERSION);
 }
 
 static struct option opts[] = {
@@ -46,11 +47,19 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 		char *end;
 	case '1':
 		check_inverse(optarg, &invert, &optind, 0);
+#ifdef KERNEL_64_USERSPACE_32
+		markinfo->mark = strtoull(optarg, &end, 0);
+		if (*end == '/') {
+			markinfo->mask = strtoull(end+1, &end, 0);
+		} else
+			markinfo->mask = 0xffffffffffffffffULL;
+#else
 		markinfo->mark = strtoul(optarg, &end, 0);
 		if (*end == '/') {
 			markinfo->mask = strtoul(end+1, &end, 0);
 		} else
 			markinfo->mask = 0xffffffff;
+#endif
 		if (*end != '\0' || end == optarg)
 			exit_error(PARAMETER_PROBLEM, "Bad MARK value `%s'", optarg);
 		if (invert)
@@ -64,17 +73,25 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 	return 1;
 }
 
+#ifdef KERNEL_64_USERSPACE_32
 static void
-print_mark(unsigned long mark, unsigned long mask, int invert, int numeric)
+print_mark(unsigned long long mark, unsigned long long mask, int numeric)
+{
+	if(mask != 0xffffffffffffffffULL)
+		printf("0x%llx/0x%llx ", mark, mask);
+	else
+		printf("0x%llx ", mark);
+}
+#else
+static void
+print_mark(unsigned long mark, unsigned long mask, int numeric)
 {
-	if (invert)
-		fputc('!', stdout);
-
 	if(mask != 0xffffffff)
 		printf("0x%lx/0x%lx ", mark, mask);
 	else
 		printf("0x%lx ", mark);
 }
+#endif
 
 /* Final check; must have specified --mark. */
 static void
@@ -91,27 +108,34 @@ print(const struct ip6t_ip6 *ip,
       const struct ip6t_entry_match *match,
       int numeric)
 {
+	struct ip6t_mark_info *info = (struct ip6t_mark_info *)match->data;
+
 	printf("MARK match ");
-	print_mark(((struct ip6t_mark_info *)match->data)->mark,
-		  ((struct ip6t_mark_info *)match->data)->mask,
-		  ((struct ip6t_mark_info *)match->data)->invert, numeric);
+
+	if (info->invert)
+		printf("!");
+	
+	print_mark(info->mark, info->mask, numeric);
 }
 
 /* Saves the union ip6t_matchinfo in parsable form to stdout. */
 static void
 save(const struct ip6t_ip6 *ip, const struct ip6t_entry_match *match)
 {
+	struct ip6t_mark_info *info = (struct ip6t_mark_info *)match->data;
+
+	if (info->invert)
+		printf("! ");
+	
 	printf("--mark ");
-	print_mark(((struct ip6t_mark_info *)match->data)->mark,
-		  ((struct ip6t_mark_info *)match->data)->mask,
-		  ((struct ip6t_mark_info *)match->data)->invert, 0);
+	print_mark(info->mark, info->mask, 0);
 }
 
 static
 struct ip6tables_match mark
 = { NULL,
     "mark",
-    NETFILTER_VERSION,
+    IPTABLES_VERSION,
     IP6T_ALIGN(sizeof(struct ip6t_mark_info)),
     IP6T_ALIGN(sizeof(struct ip6t_mark_info)),
     &help,
